@@ -15,14 +15,17 @@
 #    with this program; if not, write to the Free Software Foundation, Inc.,
 #    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
+"""
+This module holds the ApplicationInstance class, used to handle
+the one application instance mechanism 
+"""
 
 import os
 import fcntl
-import errno
-
 import logger
 import tools
 
+
 class ApplicationInstance:
     """
     Class used to handle one application instance mechanism.
@@ -35,11 +38,12 @@ class ApplicationInstance:
                             is checking at the same time
     """
 
-    def __init__(self, pidFile, autoExit = True, flock = False):
+    def __init__(self, pidFile, autoExit=True, flock=False):
         self.pidFile = pidFile
         self.pid = 0
         self.procname = ''
         self.flock = None
+
         if flock:
             self.flockExclusiv()
 
@@ -50,42 +54,43 @@ class ApplicationInstance:
     def __del__(self):
         self.flockUnlock()
 
-    def check(self, autoExit = False):
+    def check(self, autoExit=False):
         """
         Check if the current application is already running
 
         Args:
-            autoExit (bool):   automatically call sys.exit if there is an other
-                                instance running
+            autoExit (bool): automatically call sys.exit if there is an other
+                             instance running
 
         Returns:
-            bool:               ``True`` if this is the only application
-                                instance
+            bool: ``True`` if this is the only application instance
         """
-        #check if the pidfile exists
+        # check if the pidfile exists
         if not os.path.isfile(self.pidFile):
             return True
 
         self.pid, self.procname = self.readPidFile()
 
-        #check if the process with specified by pid exists
+        # check if the process with specified by pid exists
         if 0 == self.pid:
             return True
 
         if not tools.processAlive(self.pid):
             return True
 
-        #check if the process has the same procname
-        #check cmdline for backwards compatibility
+        # check if the process has the same procname
+        # check cmdline for backwards compatibility
         if self.procname and                                    \
            self.procname != tools.processName(self.pid) and    \
            self.procname != tools.processCmdline(self.pid):
-                return True
+            return True
 
         if autoExit:
-            #exit the application
+            # exit the application
             print("The application is already running !")
-            exit(0) #exit raise an exception so don't put it in a try/except block
+
+            # exit raise an exception so don't put it in a try/except block
+            exit(0)
 
         return False
 
@@ -108,8 +113,11 @@ class ApplicationInstance:
         try:
             with open(self.pidFile, 'wt') as f:
                 f.write('{}\n{}'.format(pid, procname))
+
         except OSError as e:
-            logger.error('Failed to write PID file %s: [%s] %s' %(e.filename, e.errno, e.strerror))
+            logger.error(
+                'Failed to write PID file %s: [%s] %s'
+                % (e.filename, e.errno, e.strerror))
 
         self.flockUnlock()
 
@@ -130,8 +138,10 @@ class ApplicationInstance:
         try:
             self.flock = open(self.pidFile + '.flock', 'w')
             fcntl.flock(self.flock, fcntl.LOCK_EX)
+
         except OSError as e:
-            logger.error('Failed to write flock file %s: [%s] %s' %(e.filename, e.errno, e.strerror))
+            logger.error('Failed to write flock file %s: [%s] %s'
+                         % (e.filename, e.errno, e.strerror))
 
     def flockUnlock(self):
         """
@@ -141,12 +151,14 @@ class ApplicationInstance:
         if self.flock:
             fcntl.fcntl(self.flock, fcntl.LOCK_UN)
             self.flock.close()
+
             try:
                 os.remove(self.flock.name)
             except:
-                #an other instance was faster
-                #race condition while using 'if os.path.exists(...)'
+                # an other instance was faster
+                # race condition while using 'if os.path.exists(...)'
                 pass
+
         self.flock = None
 
     def readPidFile(self):
@@ -158,31 +170,42 @@ class ApplicationInstance:
         """
         pid = 0
         procname = ''
+
         try:
             with open(self.pidFile, 'rt') as f:
                 data = f.read()
+
             data = data.split('\n', 1)
+
             if data[0].isdigit():
                 pid = int(data[0])
+
             if len(data) > 1:
                 procname = data[1].strip('\n')
+
         except OSError as e:
-            logger.warning('Failed to read PID and process name from %s: [%s] %s' %(e.filename, e.errno, e.strerror))
+            logger.warning(
+                'Failed to read PID and process name from %s: [%s] %s'
+                % (e.filename, e.errno, e.strerror))
+
         except ValueError as e:
-            logger.warning('Failed to extract PID and process name from %s: %s'
-                            %(self.pidFile, str(e)))
+            logger.warning(
+                'Failed to extract PID and process name from %s: %s'
+                % (self.pidFile, str(e)))
+
         return (pid, procname)
 
+
 if __name__ == '__main__':
     import time
 
-    #create application instance
+    # create application instance
     appInstance = ApplicationInstance('/tmp/myapp.pid')
 
-    #do something here
+    # do something here
     print("Start MyApp")
-    time.sleep(5)	#sleep 5 seconds
+    time.sleep(5)  # sleep 5 seconds
     print("End MyApp")
 
-    #remove pid file
+    # remove pid file
     appInstance.exitApplication()
