@@ -1,5 +1,6 @@
 #    Back In Time
-#    Copyright (C) 2008-2022 Oprea Dan, Bart de Koning, Richard Bailey, Germar Reitze
+#    Copyright (C) 2008-2022 Oprea Dan, Bart de Koning, Richard Bailey,
+#    Germar Reitze
 #
 #    This program is free software; you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License as published by
@@ -15,6 +16,18 @@
 #    with this program; if not, write to the Free Software Foundation, Inc.,
 #    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
+"""Configuration logic.
+
+This module and its `Config` class contain the appliation logic handling the
+configuration of Back In Time. The handling of the configuration file itself
+is separated in the module :py:module:`configfile`.
+
+Development notes:
+    Some of the methods have code comments starting with `#? ` instead of
+    `# `. These special comments are used to generate the manpage
+    `backintime-config`. The script `create-manpage-backintime-config.py`
+    parses this module for that.
+"""
 
 import os
 import sys
@@ -32,14 +45,16 @@ except ImportError:
 import tools
 import configfile
 import logger
-import mount
 import sshtools
 import encfstools
 import password
 import pluginmanager
-from exceptions import PermissionDeniedByPolicy, InvalidChar, InvalidCmd, LimitExceeded
+from exceptions import PermissionDeniedByPolicy, \
+                       InvalidChar, \
+                       InvalidCmd, \
+                       LimitExceeded
 
-_=gettext.gettext
+_ = gettext.gettext
 
 gettext.bindtextdomain('backintime', os.path.join(tools.sharePath(), 'locale'))
 gettext.textdomain('backintime')
@@ -47,9 +62,12 @@ gettext.textdomain('backintime')
 
 class Config(configfile.ConfigFileWithProfiles):
     APP_NAME = 'Back In Time'
-    VERSION = '1.3.2'
-    COPYRIGHT = 'Copyright (C) 2008-2022 Oprea Dan, Bart de Koning, Richard Bailey, Germar Reitze'
+    VERSION = '1.3.3-dev'
+    COPYRIGHT = 'Copyright (C) 2008-2022 Oprea Dan, Bart de Koning, ' \
+                'Richard Bailey, Germar Reitze'
+
     CONFIG_VERSION = 6
+    """Latest or highest possible version of Backin Time's config file."""
 
     NONE = 0
     AT_EVERY_BOOT = 1
@@ -74,53 +92,56 @@ class Config(configfile.ConfigFileWithProfiles):
     DISK_UNIT_GB = 20
 
     SCHEDULE_MODES = {
-                NONE : _('Disabled'),
-                AT_EVERY_BOOT : _('At every boot/reboot'),
+                NONE: _('Disabled'),
+                AT_EVERY_BOOT: _('At every boot/reboot'),
                 _5_MIN: _('Every 5 minutes'),
                 _10_MIN: _('Every 10 minutes'),
                 _30_MIN: _('Every 30 minutes'),
-                _1_HOUR : _('Every hour'),
-                _2_HOURS : _('Every 2 hours'),
-                _4_HOURS : _('Every 4 hours'),
-                _6_HOURS : _('Every 6 hours'),
-                _12_HOURS : _('Every 12 hours'),
-                CUSTOM_HOUR : _('Custom Hours'),
-                DAY : _('Every Day'),
-                REPEATEDLY : _('Repeatedly (anacron)'),
-                UDEV : _('When drive get connected (udev)'),
-                WEEK : _('Every Week'),
-                MONTH : _('Every Month'),
-                YEAR : _('Every Year')
+                _1_HOUR: _('Every hour'),
+                _2_HOURS: _('Every 2 hours'),
+                _4_HOURS: _('Every 4 hours'),
+                _6_HOURS: _('Every 6 hours'),
+                _12_HOURS: _('Every 12 hours'),
+                CUSTOM_HOUR: _('Custom Hours'),
+                DAY: _('Every Day'),
+                REPEATEDLY: _('Repeatedly (anacron)'),
+                UDEV: _('When drive get connected (udev)'),
+                WEEK: _('Every Week'),
+                MONTH: _('Every Month'),
+                YEAR: _('Every Year')
                 }
 
     REMOVE_OLD_BACKUP_UNITS = {
-                DAY : _('Day(s)'),
-                WEEK : _('Week(s)'),
-                YEAR : _('Year(s)')
+                DAY: _('Day(s)'),
+                WEEK: _('Week(s)'),
+                YEAR: _('Year(s)')
                 }
 
     REPEATEDLY_UNITS = {
-                HOUR : _('Hour(s)'),
-                DAY : _('Day(s)'),
-                WEEK : _('Week(s)'),
-                MONTH : _('Month(s)')
+                HOUR: _('Hour(s)'),
+                DAY: _('Day(s)'),
+                WEEK: _('Week(s)'),
+                MONTH: _('Month(s)')
                 }
 
-    MIN_FREE_SPACE_UNITS = { DISK_UNIT_MB : 'MiB', DISK_UNIT_GB : 'GiB' }
+    MIN_FREE_SPACE_UNITS = {
+        DISK_UNIT_MB: 'MiB',
+        DISK_UNIT_GB : 'GiB'
+    }
 
     DEFAULT_EXCLUDE = [ '.gvfs', '.cache/*', '.thumbnails*',
     '.local/share/[Tt]rash*', '*.backup*', '*~', '.dropbox*', '/proc/*',
     '/sys/*', '/dev/*', '/run/*', '/etc/mtab', '/var/cache/apt/archives/*.deb',
     'lost+found/*', '/tmp/*', '/var/tmp/*', '/var/backups/*', '.Private' ]
 
-    DEFAULT_RUN_NICE_FROM_CRON   = True
-    DEFAULT_RUN_NICE_ON_REMOTE   = False
+    DEFAULT_RUN_NICE_FROM_CRON = True
+    DEFAULT_RUN_NICE_ON_REMOTE = False
     DEFAULT_RUN_IONICE_FROM_CRON = True
     DEFAULT_RUN_IONICE_FROM_USER = False
     DEFAULT_RUN_IONICE_ON_REMOTE = False
-    DEFAULT_RUN_NOCACHE_ON_LOCAL  = False
+    DEFAULT_RUN_NOCACHE_ON_LOCAL = False
     DEFAULT_RUN_NOCACHE_ON_REMOTE = False
-    DEFAULT_SSH_PREFIX = 'PATH=/opt/bin:/opt/sbin:\$PATH'
+    DEFAULT_SSH_PREFIX = 'PATH=/opt/bin:/opt/sbin:\\$PATH'
     DEFAULT_REDIRECT_STDOUT_IN_CRON = True
     DEFAULT_REDIRECT_STDERR_IN_CRON = False
 
@@ -133,7 +154,7 @@ class Config(configfile.ConfigFileWithProfiles):
                 'ssh_encfs'     : (encfstools.EncFS_SSH,    _('SSH encrypted'),     _('SSH private key'),   _('Encryption'))
                 }
 
-    SSH_CIPHERS =  {'default':    _('Default'),
+    SSH_CIPHERS =  {'default': _('Default'),
                     'aes128-ctr': _('AES128-CTR'),
                     'aes192-ctr': _('AES192-CTR'),
                     'aes256-ctr': _('AES256-CTR'),
@@ -145,12 +166,12 @@ class Config(configfile.ConfigFileWithProfiles):
                     'cast128-cbc': _('Cast128-CBC'),
                     'aes192-cbc': _('AES192-CBC'),
                     'aes256-cbc': _('AES256-CBC'),
-                    'arcfour':    _('ARCFOUR') }
+                    'arcfour': _('ARCFOUR') }
 
     ENCODE = encfstools.Bounce()
     PLUGIN_MANAGER = pluginmanager.PluginManager()
 
-    def __init__(self, config_path = None, data_path = None):
+    def __init__(self, config_path=None, data_path=None):
         configfile.ConfigFileWithProfiles.__init__(self, _('Main profile'))
 
         self._APP_PATH = tools.backintimePath()
@@ -195,11 +216,17 @@ class Config(configfile.ConfigFileWithProfiles):
             else:
                 os.rename(old_path, self._LOCAL_CONFIG_PATH)
 
+        # Load global config file
         self.load(self._GLOBAL_CONFIG_PATH)
+
+        # Append local config file
         self.append(self._LOCAL_CONFIG_PATH)
 
-        #?Internal version of current config;;self.CONFIG_VERSION
-        currentConfigVersion = self.intValue('config.version', 5)
+        # Get the version of the config file
+        # or assume the highest config version if it isn't set.
+        currentConfigVersion \
+            = self.intValue('config.version', self.CONFIG_VERSION)
+
         if currentConfigVersion < self.CONFIG_VERSION:
             # config.version value wasn't stored since BiT version 0.9.99.22
             # until version 1.2.0 because of a bug. So we can't really tell
@@ -430,9 +457,9 @@ class Config(configfile.ConfigFileWithProfiles):
             self.setProfileStrValue('snapshots.path', value, profile_id)
         return True
 
-    def snapshotsMode(self, profile_id = None):
-        #?Use mode (or backend) for this snapshot. Look at 'man backintime'
-        #?section 'Modes'.;local|local_encfs|ssh|ssh_encfs
+    def snapshotsMode(self, profile_id=None):
+        #? Use mode (or backend) for this snapshot. Look at 'man backintime'
+        #? section 'Modes'.;local|local_encfs|ssh|ssh_encfs
         return self.profileStrValue('snapshots.mode', 'local', profile_id)
 
     def setSnapshotsMode(self, value, profile_id = None):
@@ -542,8 +569,9 @@ class Config(configfile.ConfigFileWithProfiles):
         self.setProfileStrValue('snapshots.ssh.private_key_file', value, profile_id)
 
     def sshMaxArgLength(self, profile_id = None):
-        #?Maximum argument length of commands run on remote host. This can be tested
-        #?with 'python3 /usr/share/backintime/common/sshMaxArg.py USER@HOST'.\n
+        #?Maximum command length of commands run on remote host. This can be tested
+        #?for all ssh profiles in the configuration
+        #?with 'python3 /usr/share/backintime/common/sshMaxArg.py [initial_ssh_cmd_length]'.\n
         #?0 = unlimited;0, >700
         value = self.profileIntValue('snapshots.ssh.max_arg_length', 0, profile_id)
         if value and value < 700:
@@ -831,8 +859,12 @@ class Config(configfile.ConfigFileWithProfiles):
         self.setProfileIntValue('schedule.mode', value, profile_id)
 
     def scheduleTime(self, profile_id = None):
-        #?What time the cronjob should run? Only valid for
-        #?\fIprofile<N>.schedule.mode\fR >= 20;0-24
+        #?Position-coded number with the format "hhmm" to specify the hour
+        #?and minute the cronjob should start (eg. 2015 means a quarter
+        #?past 8pm). Leading zeros can be omitted (eg. 30 = 0030).
+        #?Only valid for
+        #?\fIprofile<N>.schedule.mode\fR = 20 (daily), 30 (weekly),
+        #?40 (monthly) and 80 (yearly);0-2400
         return self.profileIntValue('schedule.time', 0, profile_id)
 
     def setScheduleTime(self, value, profile_id = None):
@@ -1153,7 +1185,19 @@ class Config(configfile.ConfigFileWithProfiles):
 
     def rsyncOptions(self, profile_id = None):
         #?rsync options. Options must be quoted e.g. \-\-exclude-from="/path/to/my exclude file"
-        return self.profileStrValue('snapshots.rsync_options.value', '', profile_id)
+        val = self.profileStrValue('snapshots.rsync_options.value', '', profile_id)
+
+        if '--old-args' in val:
+            logger.warning(
+                'Found rsync flag "--old-args". That flag will be removed '
+                'from the options because it does conflict with '
+                'the flag "-s" (also known as "--secluded-args" or '
+                '"--protected-args") which is used by Back In Time to force '
+                'the "new form of argument protection" in rsync.'
+            )
+            val = val.replace('--old-args', '')
+
+        return val
 
     def setRsyncOptions(self, enabled, value, profile_id = None):
         self.setProfileBoolValue('snapshots.rsync_options.enabled', enabled, profile_id)
@@ -1254,8 +1298,10 @@ class Config(configfile.ConfigFileWithProfiles):
     def takeSnapshotProgressFile(self, profile_id = None):
         return os.path.join(self._LOCAL_DATA_FOLDER, "worker%s.progress" % self.fileId(profile_id))
 
-    def takeSnapshotInstanceFile(self, profile_id = None):
-        return os.path.join(self._LOCAL_DATA_FOLDER, "worker%s.lock" % self.fileId(profile_id))
+    def takeSnapshotInstanceFile(self, profile_id=None):
+        return os.path.join(
+            self._LOCAL_DATA_FOLDER,
+            "worker%s.lock" % self.fileId(profile_id))
 
     def takeSnapshotUserCallback(self):
         return os.path.join(self._LOCAL_CONFIG_FOLDER, "user-callback")
@@ -1308,8 +1354,10 @@ class Config(configfile.ConfigFileWithProfiles):
     def restoreLogFile(self, profile_id = None):
         return os.path.join(self._LOCAL_DATA_FOLDER, "restore_%s.log" % self.fileId(profile_id))
 
-    def restoreInstanceFile(self, profile_id = None):
-        return os.path.join(self._LOCAL_DATA_FOLDER, "restore%s.lock" % self.fileId(profile_id))
+    def restoreInstanceFile(self, profile_id=None):
+        return os.path.join(
+            self._LOCAL_DATA_FOLDER,
+            "restore%s.lock" % self.fileId(profile_id))
 
     def lastSnapshotSymlink(self, profile_id = None):
         return os.path.join(self.snapshotsFullPath(profile_id), 'last_snapshot')
@@ -1353,10 +1401,14 @@ class Config(configfile.ConfigFileWithProfiles):
         if not self.isConfigured(profile_id):
             return False
 
-        if not os.path.isdir(self.snapshotsFullPath(profile_id)):
-            logger.error("%s does not exist"
-                         %self.snapshotsFullPath(profile_id),
-                         self)
+        path = self.snapshotsFullPath(profile_id)
+
+        if not os.path.exists(path):
+            return False
+
+        if not os.path.isdir(path):
+            # path exists, but is no dir: something's very wrong.
+            logger.error("%s is not a directory"%path, self)
             return False
 
         return True
@@ -1607,6 +1659,7 @@ class Config(configfile.ConfigFileWithProfiles):
             cmd = tools.which('nice') + ' -n19 ' + cmd
         return cmd
 
+
 if __name__ == '__main__':
     config = Config()
     print("snapshots path = %s" % config.snapshotsFullPath())
