@@ -115,14 +115,18 @@ class Mount(object):
         """
         self.config.PLUGIN_MANAGER.load(cfg = self.config)
         self.config.PLUGIN_MANAGER.mount(self.profile_id)
+
         if mode is None:
             mode = self.config.snapshotsMode(self.profile_id)
 
         if self.config.SNAPSHOT_MODES[mode][0] is None:
-            #mode doesn't need to mount
+            # mode doesn't need to mount
             return 'local'
+
         else:
-            while True:
+
+            while True:  # ???
+
                 try:
                     mounttools = self.config.SNAPSHOT_MODES[mode][0]
                     backend = mounttools(cfg = self.config,
@@ -131,12 +135,16 @@ class Mount(object):
                                          mode = mode,
                                          parent = self.parent,
                                          **kwargs)
+
                     return backend.mount(check = check)
+
                 except HashCollision as ex:
                     logger.warning(str(ex), self)
                     del backend
                     check = False
+
                     continue
+
                 break
 
     def umount(self, hash_id = None):
@@ -620,34 +628,32 @@ class MountControl(object):
 
         Folder structure in ~/.local/share/backintime/mnt/ (self.mount_root)::
 
-            |\ <pid>.lock              <=  mountprocess lock that will prevent
-            |                              different processes modifying
-            |                              mountpoints at one time
-            |
-            |\ <hash_id>/              <=  ``self.hash_id_path``
-            |            \                 will be shared by all profiles with
-            |            |                 the same mount settings
-            |            |
-            |            |\ mountpoint/<=  ``self.currentMountpoint``
-            |            |                 real mountpoint
-            |            |
-            |            |\ umount     <=  ``self.umount_info``
-            |            |                 json file with all nessesary args
-            |            |                 for unmount
-            |            |
-            |            \  locks/     <=  ``self.lock_path``
-            |                              for each process you have a
-            |                              ``<pid>.lock`` file
-            |
-            |\ <profile id>_<pid>/     <=  sym-link to the right path. return by
-            |                              config.snapshotsPath
-            |                              (can be ../mnt/<hash_id>/mount_point
-            |                              for ssh or
-            |                              ../mnt/<hash_id>/<HOST>/<SHARE> for
-            |                              fusesmb ...)
-            |
-            \ tmp_<profile id>_<pid>/ <=  sym-link for testing mountpoints in
-                                          settingsdialog
+            .
+            ├── <pid>.lock              <=  mountprocess lock that will prevent
+            │                               different processes modifying
+            │                               mountpoints at one time
+            │
+            ├── <hash_id>/              <=  ``self.hash_id_path`` will be
+            │   │                           shared by all profiles with the
+            │   │                           same mount settings
+            │   │
+            │   ├── mountpoint/         <=  ``self.currentMountpoint`` real
+            │   │                           mountpoint
+            │   │
+            │   ├── umount              <=  ``self.umount_info`` json file with
+            │   │                           all nessesary args for unmount
+            │   │
+            │   └── locks/              <=  ``self.lock_path`` for each process
+            │                               you have a ``<pid>.lock`` file
+            │
+            ├── <profile id>_<pid>/     <=  sym-link to the right path. return
+            │                               by config.snapshotsPath (can be
+            │                               ../mnt/<hash_id>/mount_point for ssh
+            │                               or ../mnt/<hash_id>/<HOST>/<SHARE>
+            │                               for fusesmb ...)
+            │
+            └── tmp_<profile id>_<pid>/ <=  sym-link for testing mountpoints
+                                            in settingsdialog
         """
         tools.mkdir(self.mount_root, 0o700)
         tools.mkdir(self.hash_id_path, 0o700)
@@ -877,22 +883,29 @@ class MountControl(object):
         """
         if not self.symlink:
             return
+
         if profile_id is None:
             profile_id = self.profile_id
+
         if hash_id is None:
             hash_id = self.hash_id
+
         if tmp_mount is None:
             tmp_mount = self.tmp_mount
-        dst = self.config.snapshotsPath(profile_id = profile_id,
-                                             mode = self.mode,
-                                             tmp_mount = tmp_mount)
+
+        dst = self.config.snapshotsPath(profile_id=profile_id,
+                                        mode=self.mode,
+                                        tmp_mount=tmp_mount)
         mountpoint = self.mountpoint(hash_id)
+
         if self.symlink_subfolder is None:
             src = mountpoint
         else:
             src = os.path.join(mountpoint, self.symlink_subfolder)
+
         if os.path.exists(dst):
             os.remove(dst)
+
         os.symlink(src, dst)
 
     def removeSymlink(self, profile_id = None, tmp_mount = None):
