@@ -24,65 +24,96 @@ from threading import Thread
 from test import generic
 
 sys.path.append(os.path.join(os.path.dirname(__file__), ".."))
+
 from applicationinstance import ApplicationInstance
 import tools
 
+
 class TestApplicationInstance(generic.TestCase):
+    """
+    """
+
     def setUp(self):
+        """Preparing unittests including the instanciation of an
+        ``ÀpplicationInstance``.
+        """
         super(TestApplicationInstance, self).setUp()
+
         self.temp_file = '/tmp/temp.txt'
         self.file_name = "/tmp/file_with_pid"
-        self.inst = ApplicationInstance(os.path.abspath(self.file_name), False)
+
+        self.app_instance = ApplicationInstance(
+            pidFile=os.path.abspath(self.file_name),
+            autoExit=False)
+
         self.subproc = None
 
     def tearDown(self):
+        """Delete temporary files and kill subprocesses.
+        """
         super(TestApplicationInstance, self).tearDown()
+
         for f in (self.temp_file, self.file_name):
             if os.path.exists(f):
                 os.remove(f)
-        self.killProcess()
 
-    def createProcess(self):
+        self._killProcess()
+
+    def _createProcess(self):
+        """Start a shell script and return ins PID."""
+
+        # path to the shell script
         dummyPath = os.path.join(os.path.dirname(__file__), generic.DUMMY)
+
         self.subproc = subprocess.Popen(dummyPath)
+
         return self.subproc.pid
 
-    def killProcess(self):
+    def _killProcess(self):
         if self.subproc:
             self.subproc.kill()
             self.subproc.wait()
             self.subproc = None
 
     def test_create_and_remove_pid_file(self):
-        #create pid file
-        self.inst.startApplication()
+        # create pid file
+        self.app_instance.startApplication()
         self.assertIsFile(self.file_name)
 
-        #remove pid file
-        self.inst.exitApplication()
+        # remove pid file
+        self.app_instance.exitApplication()
         self.assertIsNoFile(self.file_name)
 
-    def test_write_pid_file(self):
-        self.inst.startApplication()
+    def test_pid_file_content(self):
+        """Content of pid file fits to current process."""
+        self.app_instance.startApplication()
 
-        #get pid/procname of current process
         this_pid = os.getpid()
         this_procname = tools.processName(this_pid)
+        expected_file_content = f'{this_pid}\n{this_procname}'
 
         with open(self.file_name, 'rt') as file_with_pid:
-            self.assertEqual(file_with_pid.read(), '{}\n{}'.format(this_pid, this_procname))
+            pid_file_content = file_with_pid.read()
+
+        self.assertEqual(pid_file_content, expected_file_content)
 
     @patch('builtins.open')
     def test_write_pid_fail(self, mock_open):
+        """The test is not clear. Because of the OSError() a log error will be
+        generated. But this isn't tested here.
+
+        I assume the expected behavior is just that nothing bad happens
+        because the OSError was caught.
+        """
         mock_open.side_effect = OSError()
-        self.inst.startApplication()
+        self.app_instance.startApplication()
 
     def test_existing_process_with_correct_procname(self):
         """
         Test the check function with an existing process with correct process
         name
         """
-        pid = self.createProcess()
+        pid = self._createProcess()
         procname = tools.processName(pid)
 
         # create file with pid and process name
@@ -91,15 +122,18 @@ class TestApplicationInstance(generic.TestCase):
             file_with_pid.write(procname)
 
         # Execute test
-        self.assertFalse(self.inst.check())
-        self.assertTrue(self.inst.busy())
+        self.assertFalse(self.app_instance.check())
+        self.assertTrue(self.app_instance.busy())
 
     def test_existing_process_with_correct_proc_cmdline(self):
         """
         Test the check function with an existing process with correct process
         cmdline (for backwards compatibility)
         """
-        pid = self.createProcess()
+
+        # start an extern shell script
+        pid = self._createProcess()
+
         procname = tools.processCmdline(pid)
 
         # create file with pid and process name
@@ -108,17 +142,17 @@ class TestApplicationInstance(generic.TestCase):
             file_with_pid.write(procname)
 
         # Execute test
-        self.assertFalse(self.inst.check())
+        self.assertFalse(self.app_instance.check())
 
     def test_no_pid_file(self):
-        self.assertTrue(self.inst.check())
+        self.assertTrue(self.app_instance.check())
 
     def test_existing_process_with_wrong_procname(self):
         """
         Test the check function with an existing process with wrong process
         name
         """
-        pid = self.createProcess()
+        pid = self._createProcess()
         procname = tools.processName(pid)
 
         # create file with pid and process name
@@ -127,13 +161,13 @@ class TestApplicationInstance(generic.TestCase):
             file_with_pid.write(procname + "DELETE")
 
         # Execute test
-        self.assertTrue(self.inst.check())
+        self.assertTrue(self.app_instance.check())
 
     def test_existing_process_with_wrong_pid(self):
         """
         Test the check function with an existing process with wrong pid
         """
-        pid = self.createProcess()
+        pid = self._createProcess()
         procname = tools.processName(pid)
 
         # create file with pid and process name
@@ -142,14 +176,14 @@ class TestApplicationInstance(generic.TestCase):
             file_with_pid.write(procname)
 
         # Execute test
-        self.assertTrue(self.inst.check())
+        self.assertTrue(self.app_instance.check())
 
     def test_killing_existing_process(self):
         """
         Test the check function with an existing process with correct process
         name
         """
-        pid = self.createProcess()
+        pid = self._createProcess()
         procname = tools.processName(pid)
 
         # create file with pid and process name
@@ -157,12 +191,12 @@ class TestApplicationInstance(generic.TestCase):
             file_with_pid.write(str(pid) + "\n")
             file_with_pid.write(procname)
 
-        self.assertFalse(self.inst.check())
+        self.assertFalse(self.app_instance.check())
 
-        self.killProcess()
+        self._killProcess()
 
         # Execute test
-        self.assertTrue(self.inst.check())
+        self.assertTrue(self.app_instance.check())
 
     def test_non_existing_process(self):
         """ Test the check function with a non existing process """
@@ -173,12 +207,12 @@ class TestApplicationInstance(generic.TestCase):
             file_with_pid.write("FAKE_PROCNAME")
 
         # Execute test
-        self.assertTrue(self.inst.check())
+        self.assertTrue(self.app_instance.check())
 
     def test_leftover_empty_lockfile(self):
         with open(self.file_name, 'wt')as f:
             pass
-        self.assertTrue(self.inst.check())
+        self.assertTrue(self.app_instance.check())
 
     def write_after_flock(self, pid_file):
         inst = ApplicationInstance(os.path.abspath(pid_file),
@@ -198,13 +232,13 @@ class TestApplicationInstance(generic.TestCase):
             self.assertEqual(f.read(), 'foo')
 
     def test_flock_exclusive(self):
-        self.inst.flockExclusiv()
+        self.app_instance.flockExclusiv()
         thread = Thread(target = self.write_after_flock, args = (self.file_name,))
         thread.start()
         #give the thread some time
         thread.join(0.01)
         self.assertNotExists(self.temp_file)
-        self.inst.flockUnlock()
+        self.app_instance.flockUnlock()
         #wait for the thread to finish
         thread.join()
         self.assertExists(self.temp_file)
@@ -214,10 +248,10 @@ class TestApplicationInstance(generic.TestCase):
     @patch('builtins.open')
     def test_flock_exclusive_fail(self, mock_open):
         mock_open.side_effect = OSError()
-        self.inst.flockExclusiv()
+        self.app_instance.flockExclusiv()
 
     def test_auto_flock(self):
-        self.inst = ApplicationInstance(os.path.abspath(self.file_name),
+        self.app_instance = ApplicationInstance(os.path.abspath(self.file_name),
                                         autoExit = False,
                                         flock = True)
         thread = Thread(target = self.write_after_flock, args = (self.file_name,))
@@ -225,7 +259,7 @@ class TestApplicationInstance(generic.TestCase):
         #give the thread some time
         thread.join(0.01)
         self.assertNotExists(self.temp_file)
-        self.inst.startApplication()
+        self.app_instance.startApplication()
         #wait for the thread to finish
         thread.join()
         self.assertExists(self.temp_file)
@@ -233,7 +267,7 @@ class TestApplicationInstance(generic.TestCase):
             self.assertEqual(f.read(), 'foo')
 
     def test_autoExit_unique_process(self):
-        self.inst = ApplicationInstance(os.path.abspath(self.file_name),
+        self.app_instance = ApplicationInstance(os.path.abspath(self.file_name),
                                         autoExit = True)
 
         self.assertExists(self.file_name)
@@ -243,7 +277,7 @@ class TestApplicationInstance(generic.TestCase):
             self.assertEqual(file_with_pid.read(), '{}\n{}'.format(this_pid, this_procname))
 
     def test_autoExit_other_running_process(self):
-        pid = self.createProcess()
+        pid = self._createProcess()
         procname = tools.processName(pid)
 
         # create file with pid and process name
@@ -252,24 +286,20 @@ class TestApplicationInstance(generic.TestCase):
             file_with_pid.write(procname)
 
         with self.assertRaises(SystemExit):
-            self.inst = ApplicationInstance(os.path.abspath(self.file_name),
+            self.app_instance = ApplicationInstance(os.path.abspath(self.file_name),
                                             autoExit = True)
 
     def test_readPidFile(self):
         with open(self.file_name, "wt") as f:
             f.write('123\nfoo')
-        self.assertEqual(self.inst.readPidFile(), (123, 'foo'))
+        self.assertEqual(self.app_instance.readPidFile(), (123, 'foo'))
 
         # ValueError
         with open(self.file_name, "wt") as f:
             f.write('foo\nbar')
-        self.assertEqual(self.inst.readPidFile(), (0, 'bar'))
+        self.assertEqual(self.app_instance.readPidFile(), (0, 'bar'))
 
     @patch('builtins.open')
     def test_readPidFile_fail(self, mock_open):
         mock_open.side_effect = OSError()
-        self.assertEqual(self.inst.readPidFile(), (0, ''))
-
-# Execute tests if this program is called with python TestApplicationInstance.py
-if __name__ == '__main__':
-    unittest.main()
+        self.assertEqual(self.app_instance.readPidFile(), (0, ''))
