@@ -18,6 +18,7 @@
 
 import os
 import sys
+import pathlib
 import subprocess
 import shlex
 import signal
@@ -33,13 +34,14 @@ import hashlib
 import ipaddress
 import atexit
 from datetime import datetime
-from distutils.version import StrictVersion
+from packaging.version import Version
 from time import sleep
 keyring = None
 keyring_warn = False
 try:
     if os.getenv('BIT_USE_KEYRING', 'true') == 'true' and os.geteuid() != 0:
         import keyring
+        from keyring import backend
 except:
     keyring = None
     os.putenv('BIT_USE_KEYRING', 'false')
@@ -91,7 +93,7 @@ def backintimePath(*path):
     Get path inside 'backintime' install folder.
 
     Args:
-        *path (str):    paths that should be joind to 'backintime'
+        *path (str):    paths that should be joined to 'backintime'
 
     Returns:
         str:            'backintime' child path like::
@@ -168,7 +170,8 @@ def gitRevisionAndHash():
             pass
     return (ref, hashid)
 
-def readFile(path, default = None):
+
+def readFile(path, default=None):
     """
     Read the file in ``path`` or its '.gz' compressed variant and return its
     content or ``default`` if ``path`` does not exist.
@@ -186,16 +189,21 @@ def readFile(path, default = None):
 
     try:
         if os.path.exists(path):
+
             with open(path) as f:
                 ret_val = f.read()
+
         elif os.path.exists(path + '.gz'):
+
             with gzip.open(path + '.gz', 'rt') as f:
                 ret_val = f.read()
+
     except:
         pass
 
     return ret_val
 
+
 def readFileLines(path, default = None):
     """
     Read the file in ``path`` or its '.gz' compressed variant and return its
@@ -317,6 +325,7 @@ def mkdir(path, mode = 0o755, enforce_permissions = True):
             os.chmod(path, mode)
     return os.path.isdir(path)
 
+
 def pids():
     """
     List all PIDs currently running on the system.
@@ -326,6 +335,7 @@ def pids():
     """
     return [int(x) for x in os.listdir('/proc') if x.isdigit()]
 
+
 def processStat(pid):
     """
     Get the stat's of the process with ``pid``.
@@ -339,10 +349,13 @@ def processStat(pid):
     try:
         with open('/proc/{}/stat'.format(pid), 'rt') as f:
             return f.read()
+
     except OSError as e:
-        logger.warning('Failed to read process stat from {}: [{}] {}'.format(e.filename, e.errno, e.strerror))
+        logger.warning('Failed to read process stat from {}: [{}] {}'
+                       .format(e.filename, e.errno, e.strerror))
         return ''
 
+
 def processPaused(pid):
     """
     Check if process ``pid`` is paused (got signal SIGSTOP).
@@ -354,8 +367,10 @@ def processPaused(pid):
         bool:       True if process is paused
     """
     m = re.match(r'\d+ \(.+\) T', processStat(pid))
+
     return bool(m)
 
+
 def processName(pid):
     """
     Get the name of the process with ``pid``.
@@ -367,9 +382,11 @@ def processName(pid):
         str:        name of the process
     """
     m = re.match(r'.*\((.+)\).*', processStat(pid))
+
     if m:
         return m.group(1)
 
+
 def processCmdline(pid):
     """
     Get the cmdline (command that spawnd this process) of the process with
@@ -535,7 +552,7 @@ def rsyncCaps(data = None):
     matchers = [r'rsync\s*version\s*(\d\.\d)', r'rsync\s*version\s*v(\d\.\d.\d)']
     for matcher in matchers:
         m = re.match(matcher, data)
-        if m and StrictVersion(m.group(1)) >= StrictVersion('3.1'):
+        if m and Version(m.group(1)) >= Version('3.1'):
             caps.append('progress2')
             break
 
@@ -549,10 +566,11 @@ def rsyncCaps(data = None):
         caps.extend([i.strip(' \n') for i in line.split(',') if i.strip(' \n')])
     return caps
 
+
 def rsyncPrefix(config,
-                no_perms = True,
-                use_mode = ['ssh', 'ssh_encfs'],
-                progress = True):
+                no_perms=True,
+                use_mode=['ssh', 'ssh_encfs'],
+                progress=True):
     """
     Get rsync command and all args for creating a new snapshot. Args are
     based on current profile in ``config``.
@@ -574,15 +592,28 @@ def rsyncPrefix(config,
     """
     caps = rsyncCaps()
     cmd = []
+
     if config.nocacheOnLocal():
         cmd.append('nocache')
+
     cmd.append('rsync')
-    cmd.extend(('--recursive',     # recurse into directories
-                '--times',          # preserve modification times
-                '--devices',        # preserve device files (super-user only)
-                '--specials',       # preserve special files
-                '--hard-links',     # preserve hard links
-                '--human-readable'))# numbers in a human-readable format
+
+    cmd.extend((
+        # recurse into directories
+        '--recursive',
+        # preserve modification times
+        '--times',
+        # preserve device files (super-user only)
+        '--devices',
+        # preserve special files
+        '--specials',
+        # preserve hard links
+        '--hard-links',
+        # numbers in a human-readable format
+        '--human-readable',
+        # use "new" argument protection
+        '-s'
+    ))
 
     if config.useChecksum() or config.forceUseChecksum:
         cmd.append('--checksum')
@@ -616,7 +647,7 @@ def rsyncPrefix(config,
                     '--no-inc-recursive'))
 
     if config.bwlimitEnabled():
-        cmd.append('--bwlimit=%d' %config.bwlimit())
+        cmd.append('--bwlimit=%d' % config.bwlimit())
 
     if config.rsyncOptionsEnabled():
         cmd.extend(shlex.split(config.rsyncOptions()))
@@ -624,40 +655,53 @@ def rsyncPrefix(config,
     cmd.extend(rsyncSshArgs(config, use_mode))
     return cmd
 
-def rsyncSshArgs(config, use_mode = ['ssh', 'ssh_encfs']):
+
+def rsyncSshArgs(config, use_mode=['ssh', 'ssh_encfs']):
     """
     Get SSH args for rsync based on current profile in ``config``.
 
     Args:
-        config (config.Config): current config
-        use_mode (list):        if current mode is in this list add additional
-                                args for that mode
+        config (config.Config): Current config instance.
+        use_mode (list):        If the profiles current mode is in this list
+                                add additional args.
 
     Returns:
-        list:                   SSH args for rsync
+        list:                   List of rsync args related to SSH.
     """
+
     cmd = []
+
     mode = config.snapshotsMode()
+
     if mode in ['ssh', 'ssh_encfs'] and mode in use_mode:
-        ssh = config.sshCommand(user_host = False,
-                                 ionice = False,
-                                 nice = False)
+        ssh = config.sshCommand(user_host=False,
+                                ionice=False,
+                                nice=False)
+
         cmd.append('--rsh=' + ' '.join(ssh))
 
-        if config.niceOnRemote()     \
-          or config.ioniceOnRemote() \
-          or config.nocacheOnRemote():
+        if config.niceOnRemote() \
+           or config.ioniceOnRemote() \
+           or config.nocacheOnRemote():
+
             rsync_path = '--rsync-path='
+
             if config.niceOnRemote():
                 rsync_path += 'nice -n 19 '
+
             if config.ioniceOnRemote():
                 rsync_path += 'ionice -c2 -n7 '
+
             if config.nocacheOnRemote():
                 rsync_path += 'nocache '
+
             rsync_path += 'rsync'
+
             cmd.append(rsync_path)
+
     return cmd
 
+
 def rsyncRemove(config, run_local = True):
     """
     Get rsync command and all args for removing snapshots with rsync.
@@ -670,7 +714,7 @@ def rsyncRemove(config, run_local = True):
     Returns:
         list:                   rsync command with all args
     """
-    cmd = ['rsync', '-a', '--delete']
+    cmd = ['rsync', '-a', '--delete', '-s']
     if run_local:
         cmd.extend(rsyncSshArgs(config))
     return cmd
@@ -804,29 +848,62 @@ def keyringSupported():
     if keyring is None:
         logger.debug('No keyring due to import error.')
         return False
-    backends = []
-    try: backends.append(keyring.backends.SecretService.Keyring)
-    except: pass
-    try: backends.append(keyring.backends.Gnome.Keyring)
-    except: pass
-    try: backends.append(keyring.backends.kwallet.Keyring)
-    except: pass
-    try: backends.append(keyring.backends.kwallet.DBusKeyring)
-    except: pass
-    try: backends.append(keyring.backend.SecretServiceKeyring)
-    except: pass
-    try: backends.append(keyring.backend.GnomeKeyring)
-    except: pass
-    try: backends.append(keyring.backend.KDEKWallet)
-    except: pass
+
+    # Determine the currently active backend
     try:
+        # get_keyring() internally calls keyring.core.init_backend()
+        # which fixes non-available backends for the first call.
+        # See related issue #1321:
+        # https://github.com/bit-team/backintime/issues/1321
         displayName = keyring.get_keyring().__module__
     except:
         displayName = str(keyring.get_keyring())
-    if backends and isinstance(keyring.get_keyring(), tuple(backends)):
+
+    logger.debug("Available keyring backends:")
+    try:
+        for b in backend.get_all_keyring():
+            logger.debug(b)
+    except Exception as e:
+        logger.debug("Available backends cannot be listed: " + repr(e))
+
+    available_backends = []
+
+    # Create a list of installed backends that BiT supports (white-listed).
+    # This is done by trying to put the meta classes ("class definitions",
+    # NOT instances of the class itself!) of the supported backends
+    # into the "backends" list
+    try: available_backends.append(keyring.backends.SecretService.Keyring)
+    except Exception as e: logger.debug("Metaclass keyring.backends.SecretService.Keyring not found: " + repr(e))
+    try: available_backends.append(keyring.backends.Gnome.Keyring)
+    except Exception as e: logger.debug("Metaclass keyring.backends.Gnome.Keyring not found: " + repr(e))
+    try: available_backends.append(keyring.backends.kwallet.Keyring)
+    except Exception as e: logger.debug("Metaclass keyring.backends.kwallet.Keyring not found: " + repr(e))
+    try: available_backends.append(keyring.backends.kwallet.DBusKeyring)
+    except Exception as e: logger.debug("Metaclass keyring.backends.kwallet.DBusKeyring not found: " + repr(e))
+    try: available_backends.append(keyring.backend.SecretServiceKeyring)
+    except Exception as e: logger.debug("Metaclass keyring.backend.SecretServiceKeyring not found: " + repr(e))
+    try: available_backends.append(keyring.backend.GnomeKeyring)
+    except Exception as e: logger.debug("Metaclass keyring.backend.GnomeKeyring not found: " + repr(e))
+    try: available_backends.append(keyring.backend.KDEKWallet)
+    except Exception as e: logger.debug("Metaclass keyring.backend.KDEKWallet not found: " + repr(e))
+    # TODO (Oct. 7, 2022): Should the ChainerBackend also be supported?
+    #                    It could solve the problem of configuring the
+    #                    used backend since it iterates over all of them
+    #                    and seems to be the default backend now.
+    #                    On the other hand it could use a non-supported
+    #                    backend without a chance for BiT to notice this.
+    # See: https://github.com/jaraco/keyring/blob/977ed03677bb0602b91f005461ef3dddf01a49f6/keyring/backends/chainer.py#L11
+    # try: backends.append(keyring.backends.chainer.ChainerBackend)
+    # except Exception as e: logger.debug("Metaclass keyring.backend. not found:" + repr(e))
+
+    logger.debug("Available supported backends: " + repr(available_backends))
+
+    if available_backends and isinstance(keyring.get_keyring(), tuple(available_backends)):
         logger.debug("Found appropriate keyring '{}'".format(displayName))
         return True
+
     logger.debug("No appropriate keyring found. '{}' can't be used with BackInTime".format(displayName))
+    # TODO (Oct. 07, 2022): Write log output indicating possible solutions for this (eg. via a URL)
     return False
 
 def password(*args):
@@ -888,13 +965,18 @@ def mountArgs(path):
         list:       mount args
     """
     mp = mountpoint(path)
+    
     with open('/etc/mtab', 'r') as mounts:
+
         for line in mounts:
             args = line.strip('\n').split(' ')
+
             if len(args) >= 2:
-                    args[1] = decodeOctalEscape(args[1])
-                    if args[1] == mp:
-                        return args
+                args[1] = decodeOctalEscape(args[1])
+
+                if args[1] == mp:
+                    return args
+
     return None
 
 def device(path):
@@ -913,8 +995,10 @@ def device(path):
         str:        device
     """
     args = mountArgs(path)
+
     if args:
         return args[0]
+
     return None
 
 def filesystem(path):
@@ -932,45 +1016,132 @@ def filesystem(path):
         return args[2]
     return None
 
+def _uuidFromDev_via_filesystem(dev):
+    """Get the UUID for the block device ``dev`` from ``/dev/disk/by-uuid`` in
+    the filesystem.
+
+    Args:
+        dev (pathlib.Path): The block device path (e.g. ``/dev/sda1``).
+
+    Returns:
+        str: The UUID or ``None`` if nothing found.
+    """
+
+
+    # /dev/disk/by-uuid
+    path_DISK_BY_UUID = pathlib.Path(DISK_BY_UUID)
+
+    if not path_DISK_BY_UUID.exists():
+        return None
+
+    # Each known uuid
+    for uuid_symlink in path_DISK_BY_UUID.glob('*'):
+
+        # Resolve the symlink (get it's target) to get the real device name
+        # and compare it with the device we are looking for
+        if dev == uuid_symlink.resolve():
+
+            # e.g. 'c7aca0a7-89ed-43f0-a4f9-c744dfe673e0'
+            return uuid_symlink.name
+
+    # Nothing found
+    return None
+
+def _uuidFromDev_via_blkid_command(dev):
+    """Get the UUID for the block device ``dev`` via the extern command
+    ``blkid``.
+
+    Hint:
+        On most systems the ``blkid`` command is available only for the
+        super-user (e.g. via ``sudo``).
+
+    Args:
+        dev (pathlib.Path): The block device path (e.g. ``/dev/sda1``).
+
+    Returns:
+        str: The UUID or ``None`` if nothing found.
+    """
+
+    # Call "blkid" command
+    try:
+        # If device does not exist, blkid will exit with a non-zero code
+        output = subprocess.check_output(['blkid', dev],
+                                        stderr = subprocess.DEVNULL,
+                                        universal_newlines=True)
+
+    except (subprocess.CalledProcessError, FileNotFoundError):
+        return None
+
+    # Parse the commands output for a UUID
+    try:
+        return re.findall(r'.*\sUUID=\"([^\"]*)\".*', output)[0]
+    except IndexError:
+        # nothing found via the regex pattern
+        pass
+
+    return None
+
+def _uuidFromDev_via_udevadm_command(dev):
+    """Get the UUID for the block device ``dev`` via the extern command
+    ``udevadm``.
+
+    Args:
+        dev (pathlib.Path): The block device path (e.g. ``/dev/sda1``).
+
+    Returns:
+        str: The UUID or ``None`` if nothing found.
+    """
+    # Call "udevadm" command
+    try:
+        output = subprocess.check_output(['udevadm', 'info', f'--name={dev}'],
+                                        stderr = subprocess.DEVNULL,
+                                        universal_newlines=True)
+
+    except (subprocess.CalledProcessError, FileNotFoundError):
+        return None
+
+    # Parse the commands output for a UUID
+    try:
+        return re.findall(r'.*?ID_FS_UUID=(\S+)', output)[0]
+    except IndexError:
+        # nothing found via the regex pattern
+        pass
+
+    return None
+
+
 def uuidFromDev(dev):
     """
     Get the UUID for the block device ``dev``.
 
     Args:
-        dev (str):  block device path
+        dev (str, pathlib.Path):  block device path
 
     Returns:
         str:        UUID
     """
-    if dev and os.path.exists(dev):
-        dev = os.path.realpath(dev)
-        if os.path.exists(DISK_BY_UUID):
-            for uuid in os.listdir(DISK_BY_UUID):
-                if dev == os.path.realpath(os.path.join(DISK_BY_UUID, uuid)):
-                    return uuid
-        else:
-            c = re.compile(b'.*\sUUID="([^"]*)".*')
-            try:
-                # If device does not exist, blkid will exit with a non-zero code
-                blkid = subprocess.check_output(['blkid', dev],
-                                                stderr = subprocess.DEVNULL)
-                uuid = c.findall(blkid)
-                if uuid:
-                    return uuid[0].decode('UTF-8')
-            except:
-                pass
 
-    c = re.compile(b'.*?ID_FS_UUID=(\S+)')
-    try:
-        udevadm = subprocess.check_output(['udevadm', 'info', '--name=%s' % dev],
-                                          stderr = subprocess.DEVNULL)
-        for line in udevadm.split():
-            m = c.match(line)
-            if m:
-                return m.group(1).decode('UTF-8')
-    except:
-        pass
-    return None
+    # handle Path objects only
+    if not isinstance(dev, pathlib.Path):
+        dev = pathlib.Path(dev)
+
+    if dev.exists():
+        dev = dev.resolve()  # when /dev/sda1 is a symlink
+
+        # Look at /dev/disk/by-uuid/
+        uuid = _uuidFromDev_via_filesystem(dev)
+        if uuid:
+            return uuid
+
+        # Try extern command "blkid"
+        uuid = _uuidFromDev_via_blkid_command(dev)
+        if uuid:
+            return uuid
+
+    # "dev" doesn't exist in the filesystem
+
+    # Try "udevadm" command at the end
+    return _uuidFromDev_via_udevadm_command(dev)
 
 def uuidFromPath(path):
     """
@@ -1234,6 +1405,8 @@ def readCrontab():
             return []
         else:
             crontab = [x.strip() for x in out.strip('\n').split('\n')]
+            if crontab == ['']:  # Fixes issue #1181 (line count of empty crontab was 1 instead of 0)
+                crontab = []
             logger.debug('Read %s lines from users crontab'
                          %len(crontab))
             return crontab
@@ -1477,22 +1650,52 @@ class UniquenessSet:
         else:
             return self.reference == (st.st_size, int(st.st_mtime))
 
+
 class Alarm(object):
     """
-    Timeout for FIFO. This does not work with threading.
+    Establish a callback function that is called after a timeout.
+
+    The implementation uses a SIGALRM signal so
+    do not call code in the callback that does not support multi-threading
+    (reentrance) or you may cause non-deterministic "random" RTEs.
     """
     def __init__(self, callback = None, overwrite = True):
+        """
+        Create a new alarm instance
+
+        Args:
+            callback: Function to call when the timer ran down
+                      (ensure calling only reentrant code).
+                      Use ``None`` toi throws a ``Timeout`` exception instead.
+            overwrite: Is it allowed to (re)start the timer
+                       even though the current timer is still running
+                       ("ticking"):
+                       ``True`` cancels the current timer (if active)
+                                and restarts with the new timout.
+                       ``False` silently ignores the start request
+                                if the current timer is still "ticking"
+        """
         self.callback = callback
         self.ticking = False
         self.overwrite = overwrite
 
     def start(self, timeout):
         """
-        Start timer
+        Start the timer (which calls the handler function
+        when the timer ran down).
+
+        The start is silently ignored if the current timer is still
+        ticking and the the attribute ``overwrite`` is ``False``.
+
+        Args:
+            timeout: timer count down in seconds
         """
         if self.ticking and not self.overwrite:
             return
         try:
+            # Warning: This code may cause non-deterministic RTEs
+            #          if the handler function calls code that does
+            #          not support reentrance (see e.g. issue #1003).
             signal.signal(signal.SIGALRM, self.handler)
             signal.alarm(timeout)
         except ValueError:
@@ -1501,7 +1704,7 @@ class Alarm(object):
 
     def stop(self):
         """
-        Stop timer before it come to an end
+        Stop timer before it comes to an end
         """
         try:
             signal.alarm(0)
@@ -1511,7 +1714,11 @@ class Alarm(object):
 
     def handler(self, signum, frame):
         """
-        Timeout occur.
+        This method is called after the timer ran down to zero
+        and calls the callback function of the alarm instance.
+
+        Raises:
+            Timeout: If no callback function was set for the alarm instance
         """
         self.ticking = False
         if self.callback is None:
@@ -1701,7 +1908,8 @@ class ShutDown(object):
                                 universal_newlines = True)
         unity_version = proc.communicate()[0]
         m = re.match(r'unity ([\d\.]+)', unity_version)
-        return m and StrictVersion(m.group(1)) >= StrictVersion('7.0') and processExists('unity-panel-service')
+
+        return m and Version(m.group(1)) >= Version('7.0') and processExists('unity-panel-service')
 
 class SetupUdev(object):
     """
@@ -1722,12 +1930,19 @@ class SetupUdev(object):
             conn = bus.get_object(SetupUdev.CONNECTION, SetupUdev.OBJECT)
             self.iface = dbus.Interface(conn, SetupUdev.INTERFACE)
         except dbus.exceptions.DBusException as e:
-            if e._dbus_error_name in ('org.freedesktop.DBus.Error.NameHasNoOwner',
-                                      'org.freedesktop.DBus.Error.ServiceUnknown',
-                                      'org.freedesktop.DBus.Error.FileNotFound'):
-                conn = None
-            else:
-                raise
+            # Only DBusExceptions are  handled to do a "graceful recovery"
+            # by working without a serviceHelper D-Bus connection...
+            # All other exceptions are still raised causing BiT
+            # to stop during startup.
+            # if e._dbus_error_name in ('org.freedesktop.DBus.Error.NameHasNoOwner',
+            #                           'org.freedesktop.DBus.Error.ServiceUnknown',
+            #                           'org.freedesktop.DBus.Error.FileNotFound'):
+            logger.warning("Failed to connect to Udev serviceHelper daemon via D-Bus: " + e.get_dbus_name())
+            logger.warning("D-Bus message: " + e.get_dbus_message())
+            logger.warning("Udev-based profiles cannot be changed or checked due to Udev serviceHelper connection failure")
+            conn = None
+            # else:
+            #     raise
         self.isReady = bool(conn)
 
     def addRule(self, cmd, uuid):
@@ -2059,9 +2274,12 @@ class Daemon:
 
     def daemonize(self):
         """
-        do the UNIX double-fork magic, see Stevens' "Advanced
-        Programming in the UNIX Environment" for details (ISBN 0201563177)
-        http://www.erlenstar.demon.co.uk/unix/faq_2.html#SEC16
+        "Converts" the current process into a daemon
+        (= process running in the background)
+        and sends a SIGTERM signal to the current process.
+        This is done via the UNIX double-fork magic, see Stevens'
+        "Advanced Programming in the UNIX Environment" for details (ISBN 0201563177)
+        and this explanation: https://stackoverflow.com/a/6011298
         """
         try:
             pid = os.fork()
@@ -2100,6 +2318,7 @@ class Daemon:
         fdDup(self.stderr, sys.stderr, 'w')
 
         signal.signal(signal.SIGTERM, self.cleanupHandler)
+
         if self.pidfile:
             atexit.register(self.appInstance.exitApplication)
             # write pidfile
